#!/usr/bin/env node
'use strict';

var _findUp = require('find-up');

var _findUp2 = _interopRequireDefault(_findUp);

var _yargs = require('yargs');

var _yargs2 = _interopRequireDefault(_yargs);

var _log = require('../lib/log');

var _log2 = _interopRequireDefault(_log);

var _build = require('./build');

var _build2 = _interopRequireDefault(_build);

var _new = require('./new');

var _new2 = _interopRequireDefault(_new);

var _watch = require('./watch');

var _watch2 = _interopRequireDefault(_watch);

var _clean = require('./clean');

var _clean2 = _interopRequireDefault(_clean);

var _init = require('./init');

var _init2 = _interopRequireDefault(_init);

var _serve = require('./serve');

var _serve2 = _interopRequireDefault(_serve);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var commands = {
  build: _build2.default,
  new: _new2.default,
  watch: _watch2.default,
  clean: _clean2.default,
  init: _init2.default,
  serve: _serve2.default
};

_yargs2.default.command('init', 'scaffold a new yarn site').command('new', 'create new content').command('build', 'build your site').command('clean', 'cleans destination folder').command('serve', 'serve your site with http-server').command('watch', 'build, serve, and watch for file changes').option('incremental', {
  description: 'only build files that have changed',
  boolean: true,
  default: true
}).option('verbose', {
  description: 'log out additional log information',
  boolean: true
}).option('version', {
  alias: 'v',
  description: 'installed version',
  boolean: true
}).help('help').default('help');

var argv = _yargs2.default.argv;

console.log('yarn\n');

if (argv.verbose) {
  _log2.default.setLogLevel('verbose');
}

if (argv.version) {
  var packageJson = void 0;
  try {
    packageJson = require(_findUp2.default.sync('package.json', {
      cwd: __dirname
    }));
  } catch (e) {/* noop */}

  console.log(packageJson.version);
} else if (argv._.length === 0) {
  _yargs2.default.showHelp('log');
  process.exit(0);
} else {
  var command = argv._[0];
  var commandHandler = commands[command];

  if (!commandHandler) {
    _log2.default.warn('Unknown command: ' + argv._.join(' '));
    console.log('');
    _yargs2.default.showHelp();
  } else {
    commandHandler(argv);
  }
}