'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

exports.default = init;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _log = require('../lib/log');

var _log2 = _interopRequireDefault(_log);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _crossSpawn = require('cross-spawn');

var _crossSpawn2 = _interopRequireDefault(_crossSpawn);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function init() {
  var destination = process.cwd();

  var questions = [{
    type: 'confirm',
    name: 'destinationOk',
    message: 'OK to create new yarn site at path: ' + destination + '?',
    default: true
  }];

  _inquirer2.default.prompt(questions).then(function (answers) {
    if (answers.destinationOk === false) {
      process.exit(1);
    }

    var scaffoldSource = require('yarn-scaffold');

    // Copy scaffold project.
    try {
      _fsExtra2.default.copySync(scaffoldSource, destination);
    } catch (e) {
      _log2.default.error('Unable to copy scaffold contents into new yarn site.');
      _log2.default.error(e);
    }

    // Create our package.json file. Grab the existing dependencies from the
    // scaffold package.json and create our clean new one.
    try {
      var scaffoldJson = require(_path2.default.join(destination, 'package.json'));

      var packageJsonData = {
        main: 'my-yarn-site',
        dependencies: scaffoldJson.dependencies
      };

      var packageJsonPath = _path2.default.join(destination, 'package.json');
      _fsExtra2.default.outputFileSync(packageJsonPath, (0, _stringify2.default)(packageJsonData));
    } catch (e) {
      _log2.default.error('Unable to create package.json file.');
      _log2.default.error(e);
    }

    // Remove un-needed files.
    ['_site', 'node_modules', 'images/.gitkeep', '.npmignore', 'scaffold.js', 'README.md'].forEach(function (dir) {
      var rmDir = _path2.default.join(destination, dir);

      try {
        _fsExtra2.default.removeSync(rmDir);
      } catch (e) {
        // Fail silently, not a big deal to not be able to remove files.
      }
    });

    var npmInstallProc;
    try {
      npmInstallProc = (0, _crossSpawn2.default)('npm', ['install' // Disable caching.
      ], {
        stdio: 'inherit',
        cwd: destination
      });
    } catch (e) {
      _log2.default.warn('Unable to run `npm install`');
    }

    npmInstallProc.on('close', function () {
      _log2.default.info('New yarn site created at ' + destination);
    });
  });
}