'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function () {
  _log2.default.setSilent(true);
  (0, _serve2.default)({
    showOutput: false
  });

  var startActivity = _activityLogger2.default.start('Starting watching files.');

  var yarn = new _lib2.default({
    // Turn on incremental building.
    incremental: true,

    // Turn off caching of templates.
    noTemplateCache: true
  });

  yarn.loadState().catch(function (e) {
    _log2.default.error(e.stack);
    throw e;
  });

  var configPath = yarn.getConfig().get('path');

  var watcher = _chokidar2.default.watch([configPath.source], {
    atomic: true
  });

  function shouldIgnorePath(path) {
    return ['/.git/', // ignore .git directory.
    configPath.plugins, configPath.themes, configPath.destination].some(function (configPath) {
      return path.indexOf(configPath) > -1;
    });
  }

  // Wait for watcher to be ready before registering other watchers.
  watcher.on('ready', function () {
    _activityLogger2.default.end(startActivity);

    console.log('');
    _log2.default.info('Ready...\n');

    function handleFsChange(event, path) {
      if (shouldIgnorePath(path)) {
        return;
      }

      _log2.default.info('Change detected at: ' + path);

      var id = _activityLogger2.default.start('Rebuilding...');

      var promise = void 0;
      // If `_config.yml` changed then re-load from fs.
      if (path.indexOf(_constants.YAML.CONFIG) > -1) {
        promise = yarn.reload();
      } else {
        promise = _bluebird2.default.resolve();
      }

      promise.then(function () {
        return yarn.readFiles(path);
      }).then(function () {
        return yarn.build();
      }).then(function () {
        _activityLogger2.default.end(id);

        var date = new Date();
        _log2.default.info('\t\tdone! (' + date.toISOString() + ')');
        console.log('');
      });
    }

    // Watch for all fs events.
    watcher.on('all', (0, _debounce2.default)(handleFsChange, 200));
  });

  // Handle when theme files change and re-build entire source to reflect new
  // theme changes.
  var themeWatcher = _chokidar2.default.watch([configPath.themes]);
  themeWatcher.on('ready', function () {

    themeWatcher.on('change', function (path) {
      _log2.default.info('Theme file changed at: ' + path);
      var id = _activityLogger2.default.start('Rebuilding...');

      yarn.reload().then(function () {
        return yarn.build();
      }).then(function () {
        _activityLogger2.default.end(id);

        var date = new Date();
        _log2.default.info('\t\tdone! (' + date.toISOString() + ')');
        console.log('');
      });
    });
  });
};

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _activityLogger = require('activity-logger');

var _activityLogger2 = _interopRequireDefault(_activityLogger);

var _debounce = require('lodash/debounce');

var _debounce2 = _interopRequireDefault(_debounce);

var _log = require('../lib/log');

var _log2 = _interopRequireDefault(_log);

var _constants = require('../lib/constants');

var _lib = require('../lib');

var _lib2 = _interopRequireDefault(_lib);

var _chokidar = require('chokidar');

var _chokidar2 = _interopRequireDefault(_chokidar);

var _serve = require('./serve');

var _serve2 = _interopRequireDefault(_serve);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }