'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _homeOrTmp = require('home-or-tmp');

var _homeOrTmp2 = _interopRequireDefault(_homeOrTmp);

var _checksum = require('./checksum');

var _log = require('./log');

var _log2 = _interopRequireDefault(_log);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var data = {};

/**
 * Namespace of the Yarn site that we're caching. This is unique for every
 * directory so we can manage cache individually for each local Yarn install.
 * @param {string}
 */
var namespace = void 0;

function createFilename() {
  var namespace = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];

  var checksum = (0, _checksum.createChecksum)(namespace).slice(0, 10);
  return _path2.default.join(_homeOrTmp2.default, '.yarn-' + checksum + '.json');
}

var filename = createFilename(namespace);

var cache = {
  setNamespace: function setNamespace(val) {
    namespace = val;

    filename = createFilename(namespace);
  },
  load: function load() {
    process.on('exit', cache.save);

    try {
      data = JSON.parse(_fsExtra2.default.readFileSync(filename));
    } catch (err) {
      return;
    }
  },
  save: function save() {
    var content = {};

    // Add information about the namespace for this cache so it's
    // human readable.
    data['_namespace'] = namespace;

    try {
      content = (0, _stringify2.default)(data, null, '  ');
    } catch (err) {
      if (err.message === 'Invalid string length') {
        err.message = 'Cache too large so it\'s been cleared.';
        _log2.default.error(err.stack);
      } else {
        throw err;
      }
    }

    _fsExtra2.default.outputFileSync(filename, content);
  },
  clear: function clear() {
    data = {};
  },
  put: function put(cacheKey, cacheValue) {
    data[cacheKey] = cacheValue;
  },
  get: function get(cacheKey) {
    return data[cacheKey];
  }
};

exports.default = cache;