'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _isUndefined = require('lodash/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _isNumber = require('lodash/isNumber');

var _isNumber2 = _interopRequireDefault(_isNumber);

var _isString = require('lodash/isString');

var _isString2 = _interopRequireDefault(_isString);

var _sortBy = require('lodash/sortBy');

var _sortBy2 = _interopRequireDefault(_sortBy);

var _some = require('lodash/some');

var _some2 = _interopRequireDefault(_some);

var _filter = require('./filter');

var _filter2 = _interopRequireDefault(_filter);

var _page = require('./page');

var _page2 = _interopRequireDefault(_page);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_bluebird2.default.promisifyAll(_fsExtra2.default);

var CollectionBase = function () {
  /**
   * Create a Collection instance.
   * @param {string} name The name of the collection.
   * @param {Object} collectionConfig Config object from config file.
   * @param {Function} getConfig Delegate function that returns config object.
   */
  function CollectionBase(name) {
    var collectionConfig = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
    var getConfig = arguments[2];
    (0, _classCallCheck3.default)(this, CollectionBase);

    if ((0, _isString2.default)(name) === false || name.length === 0) {
      throw new Error('Collection requires a name.');
    }

    /**
     * Unique ID of this Collection, currently its given name.
     * @type {string}
     */
    this.id = name;

    /**
     * The collection name. Must be unique.
     * @type {string}
     */
    this.name = name;

    /**
     * Data accesible to templates.
     * @type {Object}
     */
    this.data = {};

    /**
     * Gets config.
     * @type {Function}
     */
    this._getConfig = getConfig;

    if (!(0, _isUndefined2.default)(collectionConfig.path)) {
      /**
       * Path where items belong within the collection.
       * @type {string}
       */
      this.path = _path2.default.resolve(this._getConfig().get('path.source'), collectionConfig.path);
    }

    if (!(0, _isUndefined2.default)(collectionConfig.template)) {
      /**
       * What template to use when rendering an item within this collection.
       * @type {string}
       */
      this.template = collectionConfig.template;
    }

    if (!(0, _isUndefined2.default)(collectionConfig.metadata)) {
      /**
       * Metadata attribute to use to find which items are within the
       * collection.
       * @type {string}
       */
      this.metadata = collectionConfig.metadata;
    }

    if (!(0, _isUndefined2.default)(collectionConfig.sort)) {
      /**
       * Sorting configuration.
       * @type {Object}
       */
      this.sort = {
        key: collectionConfig.sort.key,
        order: collectionConfig.sort.order
      };
    }

    if (!(0, _isUndefined2.default)(collectionConfig.pagination)) {
      var paginationConfig = collectionConfig.pagination;

      /**
       * Pagination information.
       * @param {Object}
       */
      this.pagination = {};

      if (!(0, _isUndefined2.default)(paginationConfig.template)) {
        /**
         * What template to use when rendering a pagination page.
         * @type {string}
         */
        this.pagination.template = paginationConfig.template;
      }

      if (!(0, _isUndefined2.default)(paginationConfig.size)) {
        if (!(0, _isNumber2.default)(paginationConfig.size)) {
          throw new Error('Pagination size must be a number');
        }

        /**
         * Size of each pagination page.
         * @type {number}
         */
        this.pagination.size = paginationConfig.size;
      }

      if (!(0, _isUndefined2.default)(paginationConfig.permalink_index)) {
        /**
         * Permalink pagination index configuration.
         * @type {string}
         */
        this.pagination.permalinkIndex = paginationConfig.permalink_index;
      }

      if (!(0, _isUndefined2.default)(paginationConfig.permalink_page)) {
        /**
         * Permalink pagination page configuration.
         * @type {string}
         */
        this.pagination.permalinkPage = paginationConfig.permalink_page;
      }
    }

    if (!(0, _isUndefined2.default)(collectionConfig.permalink)) {
      /**
       * Permalink configuration.
       * @type {string}
       */
      this.permalink = collectionConfig.permalink;
    }

    if (!(0, _isUndefined2.default)(collectionConfig.static)) {
      /**
       * Whether this is a static collection.
       * @type {boolean}
       */
      this.static = collectionConfig.static;
    }

    if (!(0, _isUndefined2.default)(collectionConfig.filter)) {
      /**
       * What filters are applied to this collection.
       * @type {Object}
       */
      this.filter = collectionConfig.filter;
    }

    /**
     * Array of CollectionPage objects.
     * @type {Array.<CollectionPage>}
     */
    this.pages = [];
  }

  /**
   * Whether a file is being filtered by the configured filters.
   * @param {File} file File object.
   * @return {boolean} Whether this file should be filtered out.
   */


  (0, _createClass3.default)(CollectionBase, [{
    key: 'isFiltered',
    value: function isFiltered(file) {
      if ((0, _isUndefined2.default)(this.filter)) {
        return false;
      }

      return (0, _some2.default)(this.filter, function (filterConfig, filterName) {
        return _filter2.default[filterName](file, filterConfig);
      });
    }

    /**
     * Populate the Collection's files via file system path or metadata attribute.
     * @param {Object.<string, Files>} files All Files.
     * @param {Object.<string, CollectionBase>} collections Object of all
     *   collections.
     * @return {CollectionBase}
     */

  }, {
    key: 'populate',
    value: function populate() {
      var files = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
      var collections = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
      // eslint-disable-line no-unused-vars
      return this;
    }

    /**
     * Create a CollectionPage instance.
     * @param {number} index Index of the page.
     * @param {string?} pageId Optionally give a custom ID for a CollectionPage.
     * @return {CollectionPage} CollectionPage instance.
     */

  }, {
    key: 'createPage',
    value: function createPage(index, pageId) {
      if (!(0, _isNumber2.default)(index)) {
        throw new Error('Must give an index when creating a CollectionPage.');
      }

      pageId = (0, _isUndefined2.default)(pageId) ? this.id : pageId;

      var page = new _page2.default(pageId, index);
      page.setGetConfig(this._getConfig);

      page.permalink = index === 0 ? this.pagination.permalinkIndex : this.pagination.permalinkPage;

      return page;
    }
  }, {
    key: '_linkPages',
    value: function _linkPages(shouldLinkPrevious, shouldLinkNext) {
      var _this = this;

      if (this.pages.length > 0) {
        this.pages.forEach(function (collectionPage, index) {
          var previous = _this.pages[index - 1];

          if (shouldLinkPrevious(previous, collectionPage)) {
            collectionPage.setPreviousPage(previous);
          }

          var next = _this.pages[index + 1];

          if (shouldLinkNext(next, collectionPage)) {
            collectionPage.setNextPage(next);
          }
        });
      }

      // Add data to template accessible object.
      this.data.pages = this.pages.map(function (page) {
        return page.data;
      });

      return this;
    }

    /**
     * Writes every page in this collection.
     * @param {Object} siteData Site wide data that is shared on all rendered
     *  pages.
     * @return {Promise}
     */

  }, {
    key: 'write',
    value: function write(siteData) {
      var _this2 = this;

      var pagePromises = [];

      // Write CollectionPage files.
      if (this.pages.length) {
        pagePromises = this.pages.map(function (collectionPage) {
          return collectionPage.write(_this2.pagination.template, siteData);
        });
      }

      return _bluebird2.default.all(pagePromises);
    }

    /**
     * Sorts files according to a sort config object.
     * @param {Array.<File>} files Array of File objects.
     * @param {Object} sortConfig Sort config object.
     * @return {Array.<file>} Sorted files.
     */

  }], [{
    key: 'sortFiles',
    value: function sortFiles(files, sortConfig) {
      if (sortConfig && sortConfig.key) {
        files = (0, _sortBy2.default)(files, sortConfig.key);

        if (sortConfig.order === 'descending') {
          files.reverse();
        }
      }

      return files;
    }
  }]);
  return CollectionBase;
}();

exports.default = CollectionBase;