'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _matches = require('lodash/matches');

var _matches2 = _interopRequireDefault(_matches);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var matchFn = void 0;

/**
 * Metadata filter. Checks if a file.data object matches all the configured
 * filter options.
 * @example
 * let filterConfig = {
 *   draft: true
 * };
 * file.data = {
 *   title: 'foo',
 *   draft: true
 * };
 * metadataFilter(filterConfig, file); // true
 * @param {File} file File we're checking.
 * @param {Object} filterConfig Filter config object.
 * @return {boolean} If the File matches the filterConfig object.
 */
function metadataFilter(file, filterConfig) {
  if (!matchFn) {
    matchFn = (0, _matches2.default)(filterConfig);
  }

  return matchFn(file.data);
}

metadataFilter.reset = function () {
  matchFn = undefined;
};

exports.default = metadataFilter;