'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _isUndefined = require('lodash/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _isNumber = require('lodash/isNumber');

var _isNumber2 = _interopRequireDefault(_isNumber);

var _isNil = require('lodash/isNil');

var _isNil2 = _interopRequireDefault(_isNil);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _cache = require('../cache');

var _cache2 = _interopRequireDefault(_cache);

var _checksum = require('../checksum');

var _plugin = require('../plugin');

var _plugin2 = _interopRequireDefault(_plugin);

var _template = require('../template');

var _url = require('../url');

var _url2 = _interopRequireDefault(_url);

var _render = require('../render');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var CollectionPage = function () {
  /**
   * Constructor for a CollectionPage.
   * @param {string} collectionId Collection ID.
   * @param {number} pageIndex Page index.
   * @constructor
   */
  function CollectionPage(collectionId, pageIndex) {
    (0, _classCallCheck3.default)(this, CollectionPage);

    if ((0, _isUndefined2.default)(collectionId)) {
      throw new Error('CollectionPage requires a collection ID as a string.');
    }

    if (!(0, _isNumber2.default)(pageIndex)) {
      throw new Error('CollectionPage requires a page index.');
    }

    /**
     * Unique ID of this CollectionPage. Comprised of its Collection's ID and
     * its page index.
     * @type {string}
     */
    this.id = collectionId + ':' + pageIndex;

    /**
     * Collection ID this page is a part of.
     * @type {string}
     */
    this._collectionId = collectionId;

    /**
     * Page index, 0-indexed.
     * @type {number}
     */
    this._index = pageIndex;

    /**
     * An array of Files that are in this page.
     * @type {Set.<string>}
     */
    this.files = new _set2.default();

    /**
     * The permalink template.
     * @type {string}
     */
    this.permalink;

    /**
     * Data accessible from template.
     * @type {Object}
     */
    this.data = (0, _create2.default)(null);

    // Current page number, 1-indexed for display purposes.
    this.data.page = this._index + 1;
  }

  (0, _createClass3.default)(CollectionPage, [{
    key: 'setGetConfig',
    value: function setGetConfig(getConfig) {
      this._getConfig = getConfig;
    }
  }, {
    key: 'setData',
    value: function setData() {
      var data = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      // Current page number.
      if (!(0, _isUndefined2.default)(data.page) && !(0, _isNumber2.default)(data.page)) {
        throw new Error('CollectionPage requires \'page\' as a number.');
      }

      // How many pages in the collection.
      if (!(0, _isUndefined2.default)(data.total_pages) && !(0, _isNumber2.default)(data.total_pages)) {
        throw new Error('CollectionPage requires \'total_pages\' as a number.');
      }

      // Posts displayed per page.
      if (!(0, _isUndefined2.default)(data.per_page) && !(0, _isNumber2.default)(data.per_page)) {
        throw new Error('CollectionPage requires \'per_page\' as a number.');
      }

      // Total number of posts.
      if (!(0, _isUndefined2.default)(data.total) && !(0, _isNumber2.default)(data.total)) {
        throw new Error('CollectionPage requires \'total\' as a number.');
      }

      // Update content that's accessible from template.
      (0, _assign2.default)(this.data, data);

      this._calculateDestination();
    }

    /**
     * Set what files belong in this page.
     * @param {Array.<File>} files Array of files.
     */

  }, {
    key: 'setFiles',
    value: function setFiles(files) {
      var _this = this;

      if (!(0, _isUndefined2.default)(files) && !(0, _isArray2.default)(files)) {
        throw new Error('Files must be an array.');
      }

      /**
       * Array of files in this page.
       * @type {Array.<File>}
       */
      this.data.files = files.map(function (file) {
        _this.files.add(file);

        file.collectionPages.add(_this);

        return file.data;
      });
    }

    /**
     * Generate the checksum of this CollectionPage. It's derived from the Files
     * that exist in this collection.
     * @return {string}
     */

  }, {
    key: 'getChecksum',
    value: function getChecksum() {
      var fileChecksums = [];
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(this.files), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var file = _step.value;

          fileChecksums.push(file.checksum);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      fileChecksums.sort();

      var checksum = (0, _checksum.createChecksum)(fileChecksums.join(''));

      return checksum;
    }

    /**
     * Link this page with its previus page, for use in templates.
     * @param {CollectionPage} previous CollectionPage instance.
     */

  }, {
    key: 'setPreviousPage',
    value: function setPreviousPage(previous) {
      (0, _assign2.default)(this.data, {
        // Previous page number. 0 if the current page is the first.
        prev: previous.data.page,

        // The URL of previous page. '' if the current page is the first.
        prev_link: previous.data.url
      });
    }

    /**
    * Link this page with its next page, for use in templates.
    * @param {CollectionPage} next CollectionPage instance.
     */

  }, {
    key: 'setNextPage',
    value: function setNextPage(next) {
      (0, _assign2.default)(this.data, {
        // Next page number. 0 if the current page is the last.
        next: next.data.page,

        // The URL of next page. '' if the current page is the last.
        next_link: next.data.url
      });
    }

    /**
     * Calculate both relative and absolute destination path for where to write
     * the file.
     * @private
     */

  }, {
    key: '_calculateDestination',
    value: function _calculateDestination() {
      // Calculate the permalink value.
      var relativeDestination = _url2.default.interpolatePermalink(this.permalink, this.data);

      var fileSystemSafeDestination = _url2.default.makeUrlFileSystemSafe(relativeDestination);

      /**
       * The URL without the domain, but with a leading slash,
       * e.g.  /2008/12/14/my-post.html
       * @type {string} url Relative path to file.
       */
      this.data.url = _url2.default.makePretty(fileSystemSafeDestination);

      /**
       * Absolute destination path.
       * @type {string} destination Absolute path to file.
       */
      this.destination = _path2.default.join(this._getConfig().get('path.destination'), fileSystemSafeDestination);
    }
  }, {
    key: 'render',
    value: function render(template, globalData) {
      var templateData = (0, _extends3.default)({}, globalData, {
        pagination: this.data
      });

      if ((0, _isNil2.default)(template)) {
        throw new Error('CollectionPage: No template given.\nData object: ' + (0, _stringify2.default)(this.data));
      }

      var result = void 0;
      try {
        result = (0, _template.renderTemplate)(template, templateData);
      } catch (e) {
        if (e.message.includes(_template.TemplateErrorMessage.NO_TEMPLATE)) {
          throw new Error('CollectionPage: Template not found \'' + template + '\'.\n' + ('Data: ' + (0, _stringify2.default)(this)));
        } else {
          throw e;
        }
      }

      return result;
    }

    /**
     * Writes a given CollectionPage to the file system.
     * @param {string} template Which template to use when rendering.
     * @param {Object} siteData Site wide data.
     * @return {Promise} Array of promises.
     */

  }, {
    key: 'write',
    value: function write(template, siteData) {
      var _this2 = this;

      var checksum = this.getChecksum();

      if (this._getConfig().get('incremental') && _cache2.default.get(this.id) === checksum) {
        return _promise2.default.resolve();
      }

      return (0, _render.renderAndWriteFileWithPlugins)(this, template, siteData, _plugin2.default.Event.page.beforeRender, _plugin2.default.Event.page.afterRender).then(function (result) {
        // Save checksum to cache for incremental builds.
        _cache2.default.put(_this2.id, checksum);

        return result;
      });
    }
  }]);
  return CollectionPage;
}();

exports.default = CollectionPage;