'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _isUndefined = require('lodash/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _isEmpty = require('lodash/isEmpty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _values = require('lodash/values');

var _values2 = _interopRequireDefault(_values);

var _reduce = require('lodash/reduce');

var _reduce2 = _interopRequireDefault(_reduce);

var _each = require('lodash/each');

var _each2 = _interopRequireDefault(_each);

var _chunk = require('lodash/chunk');

var _chunk2 = _interopRequireDefault(_chunk);

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A collection that derives its content from the location of a file in the
 * file system.
 */
var FileSystemCollection = function (_CollectionBase) {
  (0, _inherits3.default)(FileSystemCollection, _CollectionBase);

  function FileSystemCollection(name, collectionConfig, getConfig) {
    (0, _classCallCheck3.default)(this, FileSystemCollection);

    /**
     * Array of file id's that belong in this collection.
     * @type {Object.<string, File>}
     */
    var _this = (0, _possibleConstructorReturn3.default)(this, (FileSystemCollection.__proto__ || (0, _getPrototypeOf2.default)(FileSystemCollection)).call(this, name, collectionConfig, getConfig));

    _this.files = {};

    /**
     * Array of paths to exclude from including in this collection.
     * @type {Array.<string>}
     */
    _this.excludePaths = [];
    return _this;
  }

  /**
   * Set what paths to exclude from including in this collection.
   * @param {Object.<string, CollectionBase>} collections Array of Collections.
   */


  (0, _createClass3.default)(FileSystemCollection, [{
    key: '_setExcludePaths',
    value: function _setExcludePaths(collections) {
      var _this2 = this;

      this.excludePaths = (0, _reduce2.default)(collections, function (allPaths, collection) {
        // Only include a collection path if it exists and isn't the app source,
        // and isn't this collection's path.
        if (collection.path && collection.path !== _this2.path && collection.path !== _this2._getConfig().get('path.source')) {
          allPaths.push(collection.path);
        }

        return allPaths;
      }, []);
    }

    /**
     * Is this file's path included in this collection's excludePaths.
     * @param {File} file File object.
     * @return {boolean} true if the file's path includes an exclude path.
     * @private
     */

  }, {
    key: '_isFileExcluded',
    value: function _isFileExcluded(file) {
      return this.excludePaths.some(function (path) {
        return file.path.includes(path);
      });
    }

    /**
     * Checks to see if this file passes all requirements to be considered a part
     * of this collection.
     * @param {File} file File object.
     * @return {boolean} true if the file meets all requirements.
     */

  }, {
    key: '_isFileInCollection',
    value: function _isFileInCollection(file) {
      return file.path.includes(this.path) && !this._isFileExcluded(file) && !this.isFiltered(file);
    }

    /**
     * Add a file to the collection.
     * @param {File} file File object.
     * @return {boolean} True if the file was added to the collection.
     */

  }, {
    key: 'addFile',
    value: function addFile(file) {
      if (!this._isFileInCollection(file)) {
        return false;
      }

      // Add file.
      this.files[file.id] = file;

      // Add reference to collection.
      file.collections.add(this);

      // Set what permalink the file should use.
      file.setPermalink(this.permalink);

      // Add data to template accessible object.
      this.data.files.push(file.data);

      return true;
    }

    /**
     * Populate the Collection's files via file system path or metadata attribute.
     * @param {Object.<string, Files>} files All Files.
     * @param {?Object.<string, CollectionBase>} collections Object of all
     *   collections.
     * @return {Collection}
     */

  }, {
    key: 'populate',
    value: function populate(files, collections) {
      var _this3 = this;

      if (!(0, _isUndefined2.default)(collections)) {
        this._setExcludePaths(collections);
      }

      // Add data to template accessible object.
      this.data.files = [];

      (0, _each2.default)(files, function (file) {
        // Don't return value so we iterate over every file.
        _this3.addFile(file);
      });

      this.createCollectionPages();

      return this;
    }

    /**
     * Create CollectionPage objects for our Collection.
     * @return {boolean} True if we successfully created CollectionPages.
     */

  }, {
    key: 'createCollectionPages',
    value: function createCollectionPages() {
      var _this4 = this;

      // If no permalink paths are set then we don't render a CollectionPage.
      if (!(this.pagination && this.pagination.permalinkIndex && this.pagination.permalinkPage)) {
        return false;
      }

      if (!(0, _isEmpty2.default)(this.files)) {
        (function () {
          _this4.pages = [];

          // Sort files according to config.
          var files = _base2.default.sortFiles((0, _values2.default)(_this4.files), _this4.sort);

          // Break up our array of files into arrays that match our defined
          // pagination size.
          var pages = (0, _chunk2.default)(files, _this4.pagination.size);

          pages.forEach(function (pageFiles, index) {
            var collectionPage = _this4.createPage(index);

            // Files in the page.
            collectionPage.setFiles(pageFiles);

            // Update CollectionPage template data.
            collectionPage.setData({
              // How many pages in the collection.
              total_pages: pages.length,

              // Posts displayed per page.
              per_page: _this4.pagination.size,

              // Total number of posts.
              total: files.length
            });

            // Add to our array of pages.
            _this4.pages.push(collectionPage);
          });
        })();
      }

      this._linkPages(
      // ShouldLinkPrevious
      function (previous) {
        return previous;
      },
      // ShouldLinkNext
      function (next) {
        return next;
      });

      return true;
    }
  }]);
  return FileSystemCollection;
}(_base2.default);

exports.default = FileSystemCollection;