'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _isUndefined = require('lodash/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _chunk = require('lodash/chunk');

var _chunk2 = _interopRequireDefault(_chunk);

var _each = require('lodash/each');

var _each2 = _interopRequireDefault(_each);

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A collection that derives its content from a match in a files yaml
 * frontmatter data.
 */
var MetadataCollection = function (_CollectionBase) {
  (0, _inherits3.default)(MetadataCollection, _CollectionBase);

  function MetadataCollection(name, collectionConfig, getConfig) {
    (0, _classCallCheck3.default)(this, MetadataCollection);

    /**
     * Object which holds a mapping of metadata value to the files that contain
     * the metadata property.
     * For example with metadata of 'tags' you'd have:
     * {
     * 	'tag-name': [file, file],
     * 	'other-tag': [file, file]
     * }
     * @type {Object.<string, Array.<File>>}
     */
    var _this = (0, _possibleConstructorReturn3.default)(this, (MetadataCollection.__proto__ || (0, _getPrototypeOf2.default)(MetadataCollection)).call(this, name, collectionConfig, getConfig));

    _this.metadataFiles;
    return _this;
  }

  /**
   * Checks to see if this file passes all requirements to be considered a part
   * of this collection.
   * @param {File} file File object.
   * @return {boolean} true if the file meets all requirements.
   */


  (0, _createClass3.default)(MetadataCollection, [{
    key: '_isFileInCollection',
    value: function _isFileInCollection(file) {
      return !(0, _isUndefined2.default)(file.data[this.metadata]) && !this.isFiltered(file);
    }

    /**
     * Add a file to the collection.
     * @param {File} file File object.
     * @return {boolean} True if the file was added to the collection.
     */

  }, {
    key: 'addFile',
    value: function addFile(file) {
      var _this2 = this;

      if (!this._isFileInCollection(file)) {
        return false;
      }

      var metadataValues = file.data[this.metadata];
      if (!Array.isArray(metadataValues)) {
        metadataValues = [metadataValues];
      }

      metadataValues.forEach(function (value) {
        _this2.metadataFiles[value] = _this2.metadataFiles[value] || [];
        _this2.metadataFiles[value].push(file);

        // Add reference to collection.
        file.collections.add(_this2);

        // Add data to template accessible object.
        _this2.data.metadata[value] = _this2.data.metadata[value] || [];
        _this2.data.metadata[value].push(file.data);
      });

      return true;
    }

    /**
     * Populate the Collection's files via file system path or metadata attribute.
     * @param {Object.<string, Files>} files Object of files.
     * @return {Collection}
     */

  }, {
    key: 'populate',
    value: function populate(files) {
      var _this3 = this;

      // Create metadata files.
      this.metadataFiles = {};

      // Initialize template data.
      this.data.metadata = {};

      // Store files that are in our collection.
      (0, _each2.default)(files, function (file) {
        // Don't return value so we iterate over every file.
        _this3.addFile(file);
      });

      this.createCollectionPages();

      return this;
    }

    /**
     * Create CollectionPage objects for our Collection.
     * @return {boolean} True if we successfully created CollectionPages.
     * @private
     */

  }, {
    key: 'createCollectionPages',
    value: function createCollectionPages() {
      var _this4 = this;

      // If no permalink paths are set then we don't render a CollectionPage.
      if (!(this.pagination && this.pagination.permalinkIndex && this.pagination.permalinkPage)) {
        return false;
      }

      if (this.metadataFiles) {
        this.pages = [];

        // Create CollectionPage objects to represent our pagination pages.
        (0, _each2.default)(this.metadataFiles, function (files, metadataKey) {
          // Sort files.
          files = _base2.default.sortFiles(files, _this4.sort);

          // Break up our array of files into arrays that match our defined
          // pagination size.
          var pages = (0, _chunk2.default)(files, _this4.pagination.size);

          pages.forEach(function (pageFiles, index) {
            // Create CollectionPage.
            var collectionPage = _this4.createPage(index, _this4.id + ':' + metadataKey + ':' + index // Custom ID.
            );

            collectionPage.setData({
              // Extra template information.
              metadata: metadataKey,

              // How many pages in the collection.
              total_pages: pages.length,

              // Posts displayed per page
              per_page: _this4.pagination.size,

              // Total number of posts
              total: files.length
            });

            // Files in the page.
            collectionPage.setFiles(pageFiles);

            // Create a map of the metadataKey to its full URL on the file object.
            // Useful when rendering and wanting to link out to the metadata page.
            pageFiles.forEach(function (file) {
              file.data.metadataUrls = file.data.metadataUrls || {};
              file.data.metadataUrls[metadataKey] = collectionPage.data.url;
            });

            // Add to our array of pages.
            _this4.pages.push(collectionPage);
          });
        });
      }

      this._linkPages(
      // ShouldLinkPrevious
      function (previous, collectionPage) {
        // With metadata collections all pages aren't made in the same context.
        // i.e. for a tag metadata collection you'll have 3 pages with metadata
        // value of 'review', and 2 pages of value 'tutorial'. These different
        // metadata values should not be linked.
        return previous && _this4.metadataFiles && previous.data.metadata === collectionPage.data.metadata;
      },
      // ShouldLinkNext
      function (next, collectionPage) {
        // With metadata collections all pages aren't made in the same context.
        // i.e. for a tag metadata collection you'll have 3 pages with metadata
        // value of 'review', and 2 pages of value 'tutorial'. These different
        // metadata values should not be linked.
        return next && _this4.metadataFiles && next.data.metadata === collectionPage.data.metadata;
      });

      return true;
    }
  }]);
  return MetadataCollection;
}(_base2.default);

exports.default = MetadataCollection;