'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _findUp = require('find-up');

var _findUp2 = _interopRequireDefault(_findUp);

var _isUndefined = require('lodash/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _each = require('lodash/each');

var _each2 = _interopRequireDefault(_each);

var _constants = require('../constants');

var CONSTANTS = _interopRequireWildcard(_constants);

var _parse = require('../parse');

var _parse2 = _interopRequireDefault(_parse);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Config = function () {
  function Config() {
    (0, _classCallCheck3.default)(this, Config);

    /**
     * Full path to where we're running our app from.
     * @type {string} Full path.
     * @private
     */
    this._root;

    /**
     * Raw object that holds the config object.
     * @type {Object}
     * @private
     */
    this._raw = (0, _create2.default)(null);

    /**
     * Default values for configuration properties that must exist.
     * @type {Object}
     * @private
     */
    this._defaults = Config.defaultConfig();
  }

  /**
   * Look for a `_config.yml` file in this directory or any parent directories.
   * @return {string} Path to the local `_config.yml` file.
   */


  (0, _createClass3.default)(Config, [{
    key: 'findLocal',
    value: function findLocal() {
      // Look up directories to find a '_config.yml' file.
      var configYmlPath = _findUp2.default.sync(CONSTANTS.YAML.CONFIG);

      // If we still can't find a '_config.yml' file then throw an error.
      if (!configYmlPath) {
        throw new Error('No \'' + CONSTANTS.YAML.CONFIG + '\' file found.');
      }

      return configYmlPath;
    }

    /**
     * Find the directory where our local '_config.yml' exists.
     * @return {string} Path to the directory where our '_config.yml' file exists.
     */

  }, {
    key: 'findLocalDir',
    value: function findLocalDir() {
      return this.findLocal().replace(CONSTANTS.YAML.CONFIG, '');
    }

    /**
     * Set the root path of where we're executing from. If it's different than
     * our previous stored value then we re-load the local config.
     * @param {string} rootPath Absoute path.
     */

  }, {
    key: 'setRoot',
    value: function setRoot(rootPath) {
      var oldPath = this._root;

      // Update new root.
      this._root = rootPath;

      if (oldPath !== this._root) {
        this.loadLocal();
      }
    }
  }, {
    key: 'loadLocal',
    value: function loadLocal() {
      var localConfigPath = _path2.default.join(this._root, CONSTANTS.YAML.CONFIG);
      var localConfig = '';
      try {
        localConfig = _fs2.default.readFileSync(localConfigPath, 'utf8');
      } catch (e) {
        // noop.
      }

      var newConfig = _parse2.default.fromYaml(localConfig);

      this.update(newConfig);
    }
  }, {
    key: 'update',
    value: function update() {
      var _this = this;

      var config = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      // Store config data privately.
      this._raw = config;

      if ((0, _isUndefined2.default)(config.path)) {
        throw new Error('_config.yml requires a \'path\' value.');
      }

      // Calculate absolute path of 'paths' keys.
      this._raw.path[CONSTANTS.KEY.SOURCE] = _path2.default.resolve(this._root, this._raw.path[CONSTANTS.KEY.SOURCE]);
      (0, _each2.default)(this._raw.path, function (val, key) {
        if (key !== CONSTANTS.KEY.SOURCE) {
          _this._raw.path[key] = _path2.default.resolve(_this._raw.path.source, _this._raw.path[key]);
        }
      });
    }

    /**
     * Getter to access config properties. Everything is pushed through here
     * so we can provide required defaults if they're not set. Also enforces
     * uniform access to config properties.
     * @param {string} objectPath Path to object property, i.e. 'path.source'.
     * @return {*} Config value.
     */

  }, {
    key: 'get',
    value: function get() {
      var objectPath = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];

      var value = (0, _get2.default)(this._raw, objectPath);

      if ((0, _isUndefined2.default)(value)) {
        value = (0, _get2.default)(this._defaults, objectPath);

        if ((0, _isUndefined2.default)(value)) {
          throw new Error('Tried to access config \'' + objectPath + '\' ' + 'that does not exist.');
        }
      }

      return value;
    }

    /**
     * Reads and parses the default config YAML file from package.
     * @return {Object} Parsed default config.
     */

  }], [{
    key: 'defaultConfig',
    value: function defaultConfig() {
      return _parse2.default.fromYaml(_fs2.default.readFileSync(_path2.default.resolve(__dirname, 'defaults.yml'), 'utf8'));
    }

    /**
     * Reads and parses the example config YAML file from package.
     * @return {Object} Parsed default config.
     */

  }, {
    key: 'exampleConfig',
    value: function exampleConfig() {
      return _parse2.default.fromYaml(_fs2.default.readFileSync(_path2.default.resolve(__dirname, 'config_example.yml'), 'utf8'));
    }

    /**
     * Helper function that creates a new Config instance with the '_config.yml'
     * file already loaded.
     * @param {string} root Optional give a root path.
     * @return {Config} Config instance.
     */

  }, {
    key: 'create',
    value: function create(root) {
      var config = new Config();
      config.setRoot(root || config.findLocalDir());
      return config;
    }
  }]);
  return Config;
}();

exports.default = Config;