'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _isUndefined = require('lodash/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _merge = require('lodash/merge');

var _merge2 = _interopRequireDefault(_merge);

var _isNil = require('lodash/isNil');

var _isNil2 = _interopRequireDefault(_isNil);

var _isString = require('lodash/isString');

var _isString2 = _interopRequireDefault(_isString);

var _checksum = require('./checksum');

var _log = require('./log');

var _log2 = _interopRequireDefault(_log);

var _url = require('./url');

var _url2 = _interopRequireDefault(_url);

var _parse = require('./parse');

var _parse2 = _interopRequireDefault(_parse);

var _plugin = require('./plugin');

var _plugin2 = _interopRequireDefault(_plugin);

var _cache = require('./cache');

var _cache2 = _interopRequireDefault(_cache);

var _markdown = require('./markdown');

var _template = require('./template');

var _render = require('./render');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var File = function () {
  function File() {
    var filePath = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];
    var getConfig = arguments.length <= 1 || arguments[1] === undefined ? function () {} : arguments[1];
    (0, _classCallCheck3.default)(this, File);

    /**
     * Absolute path to file location.
     * @type {string}
     */
    this.path = filePath;

    /**
     * Unique ID for this file. Right now an alias for the file's path.
     * @type {string}
     */
    this.id = this.path;

    /**
     * Template accessible data.
     * @type {Object.<string, Object>}
     */
    this.data = (0, _create2.default)(null);

    /**
     * An array of Collections that this file belongs to.
     * @type {Set.<string>}
     */
    this.collections = new _set2.default();

    /**
     * An array of CollectionPages that this file belongs to.
     * @type {Set.<string>}
     */
    this.collectionPages = new _set2.default();

    /**
     * Get the global config object.
     * @type {Function}
     * @private
     */
    this._getConfig = getConfig;

    this.updateDataFromFileSystem();
  }

  (0, _createClass3.default)(File, [{
    key: 'updateDataFromFileSystem',
    value: function updateDataFromFileSystem() {
      /**
       * Raw contents of file, directly from file system.
       * @TODO: perhaps don't keep reference to this?
       * @type {string} One long string.
       */
      this.rawContent = _fs2.default.readFileSync(this.path, 'utf8');

      /**
       * Checksum hash of rawContent, for use in seeing if file is different.
       * @example:
       * 	'50de70409f11f87b430f248daaa94d67'
       * @type {string}
       */
      this.checksum = (0, _checksum.createChecksum)(this.rawContent);

      // Parse file's frontmatter.
      var parsedFile = _parse2.default.fromFrontMatter(this.rawContent);

      /**
       * Just the file's text content.
       * @type {string}
       */
      var content = parsedFile.content;

      /**
       * If the file itself wants to customize what its URL is then it will use
       * the `config.file.url_key` value of the File's frontmatter as the basis
       * for which the URL of this file should be.
       * So if you have a File with a frontmatter that has `url: /pandas/` then
       * the File's URL will be `/pandas/`.
       * @type {string?} url Relative path to file.
       */
      this.url = parsedFile.data[this._getConfig().get('file.url_key')];

      // Merge in new data that's accessible from template.
      (0, _merge2.default)(this.data, parsedFile.data, {
        // The content of the Page.
        content: content
      });

      this._calculateDestination();
    }

    /**
     * Path to file relative to root of project.
     * @type {string}
     */

  }, {
    key: 'setPermalink',


    /**
     * Set new permalink configuration this file should use.
     * @param {string} newPermalink New permalink.
     */
    value: function setPermalink(newPermalink) {
      this.permalink = newPermalink;
      this._calculateDestination();
    }

    /**
     * Calculate both relative and absolute destination path for where to write
     * the file.
     * @private
     */

  }, {
    key: '_calculateDestination',
    value: function _calculateDestination() {
      var destinationUrl = void 0;
      if (this.url) {
        // If the individual File defined its own unique URL that gets first
        // dibs at setting the official URL for this file.
        destinationUrl = this.url;
      } else if (this.permalink) {
        // If the file has no URL but has a permalink set on it then use it to
        // find the URL of the File.
        destinationUrl = _url2.default.interpolatePermalink(this.permalink, this.data);
      } else {
        // If the file has no URL set and no permalink then use its relative file
        // path as its url.
        destinationUrl = this.pathRelative;

        var markdownExtensions = this._getConfig().get('markdown_extension');

        // Get file extension of file. i.e. 'post.md' would give 'md'.
        var fileExtension = _path2.default.extname(destinationUrl).replace(/^\./, '');
        var index = markdownExtensions.indexOf(fileExtension);

        // Is this file's extension one of our known markdown extensions?
        if (index > -1) {
          var foundExtension = markdownExtensions[index];
          destinationUrl = destinationUrl.replace(new RegExp('.' + foundExtension + '$'), '.html');
        }
      }

      var fileSystemSafeDestination = _url2.default.makeUrlFileSystemSafe(destinationUrl);

      /**
       * Absolute destination path.
       * @type {string} destination Absolute path to file.
       */
      this.destination = _path2.default.join(this._getConfig().get('path.destination'), fileSystemSafeDestination);

      this.data.url = _url2.default.makePretty(fileSystemSafeDestination);
    }
  }, {
    key: 'render',
    value: function render(template, globalData) {
      template = (0, _isUndefined2.default)(this.data.template) ? template : this.data.template;
      var result = this.data.content;

      var templateData = (0, _extends3.default)({}, globalData, {
        file: this.data
      });

      try {
        // Set result of content to result content.
        result = (0, _template.renderTemplateString)(this.data.content, templateData);

        // Set result to file's contents.
        this.data.content = result;
      } catch (e) {
        _log2.default.error(e.message);
        throw new Error('File: Could not render file\'s contents.\n' + ('File: ' + (0, _stringify2.default)(this)));
      }

      // Convert to HTML.
      // However if the File's frontmatter sets markdown value to false then
      // skip the markdown conversion.
      if (this.data.markdown !== false) {
        result = (0, _markdown.renderMarkdown)(this.data.content);
        this.data.content = result;
      }

      if (!(0, _isNil2.default)(template) && !((0, _isString2.default)(template) && template.length === 0)) {
        try {
          result = (0, _template.renderTemplate)(template, templateData);
        } catch (e) {
          if (e.message.includes(_template.TemplateErrorMessage.NO_TEMPLATE)) {
            throw new Error('File: Template \'' + template + '\' not found.\n' + ('File: ' + (0, _stringify2.default)(this)));
          } else {
            throw e;
          }
        }
      }

      return result;
    }

    /**
     * Writes a given File object to the file system.
     * @param {Object} siteData Site wide data.
     * @return {Promise} Promise.
     */

  }, {
    key: 'write',
    value: function write(siteData) {
      var _this = this;

      if (this._getConfig().get('incremental') && _cache2.default.get(this.path) === this.checksum) {
        return _promise2.default.resolve();
      }

      var promises = [];

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(this.collections), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var collection = _step.value;

          // Don't render if we have no template defined.
          if ((0, _isUndefined2.default)(collection.template)) {
            continue;
          }

          var promise = (0, _render.renderAndWriteFileWithPlugins)(this, collection.template, siteData, _plugin2.default.Event.file.beforeRender, _plugin2.default.Event.file.afterRender);

          promises.push(promise);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return _promise2.default.all(promises).then(function (result) {
        // Save checksum to cache for incremental builds.
        _cache2.default.put(_this.path, _this.checksum);

        return result;
      });
    }
  }, {
    key: 'pathRelative',
    get: function get() {
      return this.path.replace(this._getConfig().get('path.source'), '');
    }
  }]);
  return File;
}();

exports.default = File;