'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _values = require('lodash/values');

var _values2 = _interopRequireDefault(_values);

var _each = require('lodash/each');

var _each2 = _interopRequireDefault(_each);

var _reduce = require('lodash/reduce');

var _reduce2 = _interopRequireDefault(_reduce);

var _defaults = require('lodash/defaults');

var _defaults2 = _interopRequireDefault(_defaults);

var _map = require('lodash/map');

var _map2 = _interopRequireDefault(_map);

var _isNil = require('lodash/isNil');

var _isNil2 = _interopRequireDefault(_isNil);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _log = require('./log');

var _log2 = _interopRequireDefault(_log);

var _cache = require('./cache');

var _cache2 = _interopRequireDefault(_cache);

var _constants = require('./constants');

var CONSTANTS = _interopRequireWildcard(_constants);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _url = require('./url');

var _url2 = _interopRequireDefault(_url);

var _template = require('./template');

var _markdown = require('./markdown');

var _index = require('./plugin/index');

var _index2 = _interopRequireDefault(_index);

var _theme = require('./theme');

var _theme2 = _interopRequireDefault(_theme);

var _collection = require('./collection');

var _file = require('./file');

var _file2 = _interopRequireDefault(_file);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_bluebird2.default.promisifyAll(_fsExtra2.default);

var Yarn = function () {
  /**
   * Create a new Yarn instance.
   * @param {Object} options Options object.
   * @param {string} options.rootPath Where the root path of this Yarn instance
   *   points to.
   * @param {boolean} options.incremental If we should incremental build files.
   * @param {boolean} options.noTemplateCache Should templates be cached.
   *   Typically this is only off when developing or in watch mode.
   */
  function Yarn() {
    var _this = this;

    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
    (0, _classCallCheck3.default)(this, Yarn);

    this.getConfig = function () {
      return _this.config;
    };

    /**
     * Expose config object on instance.
     * @type {Config}
     */
    this.config = new _config2.default();

    var rootPath = (0, _isNil2.default)(options.rootPath) ? this.config.findLocalDir() : options.rootPath;

    // Update root path.
    this.config.setRoot(rootPath);

    /**
     * Save options passed into instance.
     * @type {Object}
     */
    this.options = (0, _defaults2.default)(options, {
      noTemplateCache: false
    });

    /**
     * All files found in our source path.
     * Key is the full path to the file, value is the actual File object.
     * @type {Object.<string, File>}
     */
    this.files = (0, _create2.default)(null);

    /**
     * Mapping of Collection IDs to the instance.
     * @type {Object.<string, Collection>}
     */
    this.collections = (0, _create2.default)(null);

    /**
     * Class responsible for handling themes.
     * @type {Theme}
     */
    this.theme = new _theme2.default();
    this.theme.setGetConfig(this.getConfig);

    /**
     * Site wide data available in all templates.
     * @type {Object.<string, Object>}
     */
    this.data = (0, _create2.default)(null);

    /**
     * Expose collections.
     * @type {Object}
     */
    this.data.collections = (0, _create2.default)(null);

    // Kick off leading config update.
    this.updateConfig();

    _cache2.default.setNamespace(rootPath);

    // Options passed into constructor take precedence over the config value.
    // We default to loading cache, unless explicitly set to false.
    var shouldLoadCache = !(0, _isNil2.default)(options.incremental) ? options.incremental !== false : this.config.get('incremental') !== false;
    if (shouldLoadCache) {
      _cache2.default.load();
    }
  }

  /**
   * Returns our config instance.
   * @return {Object}
   */


  (0, _createClass3.default)(Yarn, [{
    key: 'updateConfig',
    value: function updateConfig() {
      var _this2 = this;

      _url2.default.setSlugOptions(this.config.get('slug'));

      this.theme.update();

      // Configure template engine.
      (0, _template.configureTemplateEngine)({
        paths: this.theme.config.path.templates,
        noCache: this.options.noTemplateCache
      });

      // Configure markdown engine.
      (0, _markdown.configureMarkdownEngine)(this.config.get('remarkable'));

      /**
       * Expose site data from config file.
       * @type {Object}
       */
      this.data.site = this.config.get('site');

      // Update our collection configs.
      (0, _map2.default)(this.config.get('collections'), function (collectionConfig, collectionName) {
        var instance = (0, _collection.createCollection)(collectionName, collectionConfig, _this2.getConfig);

        _this2.collections[instance.id] = instance;
      });
    }
  }, {
    key: 'loadPlugins',
    value: function loadPlugins() {
      var id = _log2.default.startActivity('Loading plugins.\t\t');

      // Built-in plugins.
      _index2.default.loadFromPackageJson(_url2.default.pathFromRoot('./'), this.config.get('plugins'));
      [this.theme.config.path.plugins, // Active theme plug-ins
      this.config.get('path.plugins') // Site plug-ins
      ].forEach(function (path) {
        return _index2.default.loadFromDirectory(path);
      });

      _log2.default.endActivity(id);
    }
  }, {
    key: 'readFiles',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
        var _this3 = this;

        var id, configPathSource, ignorePatterns, files;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                id = _log2.default.startActivity('Reading files.\t\t\t');
                configPathSource = this.config.get('path.source');

                // Create an array of patterns that we should ignore when reading the source
                // files of the Yarn site from disk.
                // This primarily includes the '_config.yml' file as well as every path
                // directory that isn't our source path, primarily '_site', '_plugins',
                // and '_themes'.

                ignorePatterns = (0, _values2.default)(CONSTANTS.YAML).concat((0, _values2.default)(this.config.get('path')).filter(function (path) {
                  return path !== configPathSource;
                }));

                // Ignore package.json file as well.

                ignorePatterns.push('package.json');

                // Ignore StaticCollection paths as those are not processed at all.
                (0, _each2.default)(this.collections, function (collection) {
                  if (collection.static) {
                    ignorePatterns.push(collection.path);
                  }
                });

                // Read all files from disk and get their file paths.
                _context.next = 7;
                return _bluebird2.default.fromCallback(function (cb) {
                  (0, _glob2.default)(configPathSource + '/**/*', {
                    // Do not match directories, only files.
                    nodir: true,
                    // Array of glob patterns to exclude from matching.
                    ignore: ignorePatterns.map(function (path) {
                      return '**/' + path + '/**';
                    }).concat(configPathSource + '/node_modules/**'),
                    // Follow symlinks.
                    follow: true
                  }, cb);
                });

              case 7:
                files = _context.sent;


                files.forEach(function (filePath) {
                  // Correct the filePath created by glob to be compatible with Windows.
                  // Known issue in node-glob https://github.com/isaacs/node-glob/pull/263.
                  filePath = filePath.replace(/\//g, _path2.default.sep);

                  var sourceFile = new _file2.default(filePath, _this3.getConfig);
                  _this3.files[sourceFile.id] = sourceFile;
                });

                // Populate every collection with its files.
                (0, _each2.default)(this.collections, function (collection) {
                  collection.populate(_this3.files, _this3.collections);

                  // Add collection data to our global data object.
                  _this3.data.collections[collection.name] = collection.data;
                });

                _log2.default.endActivity(id);

                return _context.abrupt('return', _bluebird2.default.resolve());

              case 12:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function readFiles() {
        return _ref.apply(this, arguments);
      }

      return readFiles;
    }()

    /**
     * Read theme files, processing any files that it needs to so that we get
     * the final path to the ready to write files.
     * @return {Proise} Promise object.
     */

  }, {
    key: 'readTheme',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
        var promise, id;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                promise = void 0;

                // Reading theme files.

                id = _log2.default.startActivity('Reading theme files.\t\t');
                _context2.prev = 2;
                _context2.next = 5;
                return this.theme.read();

              case 5:
                promise = _context2.sent;
                _context2.next = 11;
                break;

              case 8:
                _context2.prev = 8;
                _context2.t0 = _context2['catch'](2);

                _log2.default.error(_context2.t0);

              case 11:
                _log2.default.endActivity(id);

                // Expose theme data globally.
                this.data.theme = this.theme.data;

                return _context2.abrupt('return', promise);

              case 14:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[2, 8]]);
      }));

      function readTheme() {
        return _ref2.apply(this, arguments);
      }

      return readTheme;
    }()
  }, {
    key: 'loadState',
    value: function () {
      var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return this.readTheme();

              case 2:

                this.loadPlugins();

                _context3.next = 5;
                return this.readFiles();

              case 5:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function loadState() {
        return _ref3.apply(this, arguments);
      }

      return loadState;
    }()

    /**
     * Reload state of Yarn from file system.
     * @return {Promise}
     */

  }, {
    key: 'reload',
    value: function () {
      var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                this.config.loadLocal();
                this.updateConfig();
                _context4.next = 4;
                return this.loadState();

              case 4:
                return _context4.abrupt('return', _context4.sent);

              case 5:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function reload() {
        return _ref4.apply(this, arguments);
      }

      return reload;
    }()

    /**
     * Removes configured destination directory and all files contained.
     * @return {Promise} Promise object.
     */

  }, {
    key: 'cleanDestination',
    value: function () {
      var _ref5 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5() {
        var _this4 = this;

        var promise, id;
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                promise = void 0;
                id = _log2.default.startActivity('Cleaning destination.\t\t');
                _context5.prev = 2;
                _context5.next = 5;
                return _bluebird2.default.fromCallback(function (cb) {
                  (0, _rimraf2.default)(_this4.config.get('path.destination'), cb);
                });

              case 5:
                promise = _context5.sent;
                _context5.next = 11;
                break;

              case 8:
                _context5.prev = 8;
                _context5.t0 = _context5['catch'](2);

                _log2.default.error(_context5.t0);

              case 11:

                // Clear cache.
                _cache2.default.clear();

                _log2.default.endActivity(id);

                return _context5.abrupt('return', promise);

              case 14:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this, [[2, 8]]);
      }));

      function cleanDestination() {
        return _ref5.apply(this, arguments);
      }

      return cleanDestination;
    }()

    /**
     * Writes Theme files and assets to file system.
     * @return {Promise} Promise object.
     */

  }, {
    key: 'writeThemeFiles',
    value: function () {
      var _ref6 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6() {
        var promise, id;
        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                promise = void 0;

                // Write theme static files.

                id = _log2.default.startActivity('Writing theme files.\t\t');
                _context6.prev = 2;
                _context6.next = 5;
                return this.theme.write();

              case 5:
                promise = _context6.sent;
                _context6.next = 11;
                break;

              case 8:
                _context6.prev = 8;
                _context6.t0 = _context6['catch'](2);

                _log2.default.error(_context6.t0);

              case 11:
                _log2.default.endActivity(id);

                return _context6.abrupt('return', promise);

              case 13:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this, [[2, 8]]);
      }));

      function writeThemeFiles() {
        return _ref6.apply(this, arguments);
      }

      return writeThemeFiles;
    }()

    /**
     * Builds the yarn site in its entirety.
     * @return {Promise} Promise of when we're done building.
     */

  }, {
    key: 'build',
    value: function () {
      var _ref7 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee7() {
        var _this5 = this;

        var promise, fileActivityId, filePromises, collectionActivityId;
        return _regenerator2.default.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                promise = void 0;

                if (!this.config.get('clean_destination')) {
                  _context7.next = 5;
                  break;
                }

                _context7.next = 4;
                return this.cleanDestination();

              case 4:
                promise = _context7.sent;

              case 5:

                this.data.yarn = Yarn.getYarnData();

                _context7.next = 8;
                return this.writeThemeFiles();

              case 8:
                fileActivityId = _log2.default.startActivity('Writing files.\t\t\t');
                filePromises = void 0;
                _context7.prev = 10;

                filePromises = (0, _reduce2.default)(this.files, function (result, file) {
                  return result.concat(file.write(_this5.data));
                }, []);

                _context7.next = 14;
                return _bluebird2.default.all(filePromises);

              case 14:
                promise = _context7.sent;
                _context7.next = 20;
                break;

              case 17:
                _context7.prev = 17;
                _context7.t0 = _context7['catch'](10);

                _log2.default.error(_context7.t0.stack);

              case 20:
                _log2.default.endActivity(fileActivityId);

                collectionActivityId = _log2.default.startActivity('Writing collection pages.\t');
                _context7.prev = 22;
                _context7.next = 25;
                return _bluebird2.default.all((0, _map2.default)(this.collections, function (collection) {
                  return collection.write(_this5.data);
                }));

              case 25:
                promise = _context7.sent;
                _context7.next = 31;
                break;

              case 28:
                _context7.prev = 28;
                _context7.t1 = _context7['catch'](22);

                _log2.default.error(_context7.t1.stack);

              case 31:
                _log2.default.endActivity(collectionActivityId);

                return _context7.abrupt('return', promise);

              case 33:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this, [[10, 17], [22, 28]]);
      }));

      function build() {
        return _ref7.apply(this, arguments);
      }

      return build;
    }()

    /**
     * Get information about the Yarn installation from its package.json.
     * @return {Object}
     */

  }], [{
    key: 'getYarnData',
    value: function getYarnData() {
      var packageJson = {};
      try {
        packageJson = require(_url2.default.pathFromRoot('./package.json'));
      } catch (e) {/* swallow */}

      return {
        version: packageJson.version,
        time: (0, _moment2.default)(new Date().getTime()).format()
      };
    }
  }]);
  return Yarn;
}();

exports.default = Yarn;