'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.yarnPackageNameRegex = undefined;
exports.getYarnPackageNames = getYarnPackageNames;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _each = require('lodash/each');

var _each2 = _interopRequireDefault(_each);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var packageJson = 'package.json';

var packageKeys = ['dependencies', 'devDependencies', 'peerDependencies'];

/**
 * RegExp used to find yarn packages.
 * @type {RegExp}
 */
var yarnPackageNameRegex = exports.yarnPackageNameRegex = /^yarn\-/;

/**
 * Looks at a package.json file and retrieves all module names that pass our
 * yarnPackageNameRegex, effectively all packages that start with `yarn-`.
 * @param {string} directory Directory to look for a package.json file.
 * @return {Array} Array of package names found.
 */
function getYarnPackageNames() {
  var directory = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];

  var packagePath = _path2.default.join(directory, packageJson);

  var json = void 0;
  try {
    json = _fsExtra2.default.readJsonSync(packagePath);
  } catch (e) {/* swallow */}

  var moduleNames = [];

  if (!json) {
    return moduleNames;
  }

  packageKeys.forEach(function (packageKey) {
    (0, _each2.default)(json[packageKey], function (version, name) {
      // Only include a yarn package extension once.
      if (yarnPackageNameRegex.test(name) && !moduleNames.includes(name)) {
        moduleNames.push(name);
      }
    });
  });

  return moduleNames;
}