'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.renderMarkdown = renderMarkdown;
exports.configureMarkdownEngine = configureMarkdownEngine;

var _remarkable = require('remarkable');

var _remarkable2 = _interopRequireDefault(_remarkable);

var _highlight = require('highlight.js');

var _highlight2 = _interopRequireDefault(_highlight);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Methods for interacting with our Markdown engine.
 * Supports rendering Markdown text and configure the engine.
 */
var md = void 0;

/**
 * Default options used for Markdown engine.
 * @type {Object}
 */
var DEFAULT_OPTIONS = {
  highlight: function highlight(str, lang) {
    if (lang && _highlight2.default.getLanguage(lang)) {
      try {
        return _highlight2.default.highlight(lang, str).value;
      } catch (e) {/* noop */}
    }

    try {
      return _highlight2.default.highlightAuto(str).value;
    } catch (e) {} /* noop */

    // use external default escaping
    return '';
  }
};

/**
 * Render a piece of string from Markdown to HTML.
 * @param {string} str String to parse.
 * @return {string} Parsed Markdown.
 */
function renderMarkdown() {
  var str = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];

  return md.render(str);
}

/**
 * Configure our Markdown engine.
 * @param {Object} options Configuration object.
 */
function configureMarkdownEngine() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  // Lazily create markdown instance.
  if (!md) {
    md = new _remarkable2.default(options.preset);
  }

  // Create copy.
  options = (0, _assign2.default)({}, options);

  // If highlight is set to true then attach the highlight function handler.
  if (options.highlight === true) {
    options.highlight = DEFAULT_OPTIONS.highlight;
  }

  // Disable Markdown from auto converting indented lines of text to render
  // as a <code/> block. This conflicted when rendering pages so it's being
  // disabled.
  md.block.ruler.disable(['code']);

  md.set(options);
}