'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.parse = parse;
exports.stringify = stringify;

var _grayMatter = require('gray-matter');

var _grayMatter2 = _interopRequireDefault(_grayMatter);

var _jsYaml = require('js-yaml');

var _jsYaml2 = _interopRequireDefault(_jsYaml);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Interface to gray-matter for parsing files with YAML frontmatter.
 */
var frontMatterOptions = {
  parser: _jsYaml2.default.safeLoad
};

/**
 * Parse a file with front matter.
 * @param {string} str String to parse.
 * @param {Object} options Additional options.
 * @return {JSON} JSON object.
 */
function parse() {
  var str = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];
  var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  return (0, _grayMatter2.default)(str, (0, _assign2.default)({}, frontMatterOptions, options));
}

/**
 * Stringify a document.
 * @param  {string} str Content to append to YAML.
 * @param  {Object} data Data to convert to YAML and prepend to document.
 * @return {string} Content with prepended YAML data.
 */
function stringify() {
  var str = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];
  var data = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  return _grayMatter2.default.stringify(str, data);
}