'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _isUndefined = require('lodash/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _merge = require('lodash/merge');

var _merge2 = _interopRequireDefault(_merge);

var _json = require('../json');

var _event = require('./event');

var _event2 = _interopRequireDefault(_event);

var _api = require('./api');

var _api2 = _interopRequireDefault(_api);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Plugin = function () {
  function Plugin() {
    (0, _classCallCheck3.default)(this, Plugin);

    this._handlers = {};

    this._constructAPIEventHandlers(_api2.default);
  }

  (0, _createClass3.default)(Plugin, [{
    key: '_constructAPIEventHandlers',
    value: function _constructAPIEventHandlers(API) {
      var addEventHandler = this.addEventHandler.bind(this);

      // Dynamically create publicly accessible event functions.
      for (var eventCategory in _event2.default) {
        API.event[eventCategory] = {};

        var _loop = function _loop(eventName) {
          var eventKey = _event2.default[eventCategory][eventName];

          API.event[eventCategory][eventName] = function (callback) {
            addEventHandler(eventKey, callback);
          };
        };

        for (var eventName in _event2.default[eventCategory]) {
          _loop(eventName);
        }
      }

      // Expose utility merge function.
      API.merge = _merge2.default;
    }
  }, {
    key: '_reset',
    value: function _reset() {
      this._handlers = {};
    }
  }, {
    key: '_getHandlers',
    value: function _getHandlers(eventName) {
      this._handlers[eventName] = this._handlers[eventName] || [];
      return this._handlers[eventName];
    }
  }, {
    key: 'addEventHandler',
    value: function addEventHandler(eventName, handler) {
      this._getHandlers(eventName).push(handler);
    }

    /**
     * Processes our queue of event handlers for a given eventName.
     * If a handler returns 'undefined' then we give the next handler
     * the original arguments so that every handler gets to touch the
     * data.
     * @param {string} eventName Event key where the handlers live.
     * @param {*} args Arguments.
     * @return {Promise} Promise that resolves to the ending value of the
     *   promise chain.
     */

  }, {
    key: 'processEventHandlers',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(eventName) {
        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          args[_key - 1] = arguments[_key];
        }

        var handlers, processedValue, i, handler, newVal, allTypesMatch;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                handlers = this._getHandlers(eventName);
                processedValue = args;
                i = 0;

              case 3:
                if (!(i < handlers.length)) {
                  _context.next = 22;
                  break;
                }

                handler = handlers[i];
                newVal = void 0;
                _context.next = 8;
                return handler.apply(undefined, processedValue);

              case 8:
                newVal = _context.sent;


                // We allow a plugin to return undefined or null to implicitly
                // allow us to use the original value that was passed into the
                // plugin handler.
                if ((0, _isUndefined2.default)(newVal) || newVal === null) {
                  newVal = processedValue;
                }

                // If our original value was an array of 1 and is now not an
                // array because a plugin just returned the value stick it back
                // into an array to keep its type consistent.
                if (processedValue.length === 1 && !Array.isArray(newVal)) {
                  newVal = [newVal];
                }

                if (!(newVal.length !== processedValue.length)) {
                  _context.next = 15;
                  break;
                }

                throw new Error('Plugin handler for \'' + eventName + '\' did not' + ' return all arguments given to it.');

              case 15:
                allTypesMatch = newVal.every(function (val, index) {
                  return (typeof val === 'undefined' ? 'undefined' : (0, _typeof3.default)(val)) === (0, _typeof3.default)(processedValue[index]);
                });

                if (allTypesMatch) {
                  _context.next = 18;
                  break;
                }

                throw new Error('Plugin handler for \'' + eventName + '\' did not' + ' return all arguments in given order.');

              case 18:

                processedValue = newVal;

              case 19:
                i++;
                _context.next = 3;
                break;

              case 22:
                return _context.abrupt('return', processedValue.length === 1 ? processedValue[0] : processedValue);

              case 23:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function processEventHandlers(_x, _x2) {
        return _ref.apply(this, arguments);
      }

      return processEventHandlers;
    }()
  }, {
    key: '_loadPlugin',
    value: function _loadPlugin() {
      var pluginPath = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];
      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      require(pluginPath)(_api2.default, options);
    }
  }, {
    key: 'loadFromPackageJson',
    value: function loadFromPackageJson() {
      var _this = this;

      var directory = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];
      var pluginConfigs = arguments[1];

      (0, _json.getYarnPackageNames)(directory).forEach(function (name) {
        var configName = name.replace(_json.yarnPackageNameRegex, '');
        var pluginConfig = pluginConfigs[configName];

        if (pluginConfig && pluginConfig.enabled) {
          _this._loadPlugin(name, pluginConfig.options);
        }
      });
    }

    /**
     * Load all .js files from a directory and load them as a plugin.
     * @param {string} directory Path to directory where plugin files exist.
     */

  }, {
    key: 'loadFromDirectory',
    value: function loadFromDirectory() {
      var _this2 = this;

      var directory = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];

      var files = _glob2.default.sync(directory + '/**/*.js', {
        nodir: true
      });

      if (!files.length) {
        return;
      }

      files.forEach(function (filePath) {
        return _this2._loadPlugin(filePath);
      });
    }
  }]);
  return Plugin;
}();

var instance = new Plugin();

instance.Event = _event2.default;
instance.API = _api2.default;

exports.default = instance;