'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.renderAndWriteFileWithPlugins = exports.renderFileWithPlugins = exports.writeToDiskWithPlugins = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

/**
 * Wrapper for writing to the file system.
 * @param {(File|CollectionPage)} file File or CollectionPage object.
 * @param {string} content Content of file.
 * @param {string} encoding What encoding to use when writing file.
 * @return {Promise}
 */
var writeToDiskWithPlugins = exports.writeToDiskWithPlugins = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(file, content) {
    var encoding = arguments.length <= 2 || arguments[2] === undefined ? 'utf8' : arguments[2];

    var _ref2, _ref3, result;

    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return _plugin2.default.processEventHandlers(_plugin2.default.Event.collection.beforeWrite, file, content);

          case 2:
            _ref2 = _context.sent;
            _ref3 = (0, _slicedToArray3.default)(_ref2, 2);
            file = _ref3[0];
            content = _ref3[1];
            _context.next = 8;
            return _bluebird2.default.fromCallback(function (cb) {
              _fsExtra2.default.outputFile(file.destination, content, encoding, cb);
            });

          case 8:
            result = _context.sent;
            _context.next = 11;
            return _plugin2.default.processEventHandlers(_plugin2.default.Event.collection.afterWrite, file, content);

          case 11:
            return _context.abrupt('return', result);

          case 12:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function writeToDiskWithPlugins(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Write a file to the file system. Calls all plugin events.
 * @param {(File|CollectionPage)} file File or CollectionPage object.
 * @param {string} template Which template template to use.
 * @param {Object} siteData Site wide template data.
 * @param {Plugin.Event} eventBefore Which event handler to process before
 *   rendering the file.
 * @param {Plugin.Event} eventAfter Which event handler to process after
 *   rendering the file.
 * @return {Promise}
 */


var renderFileWithPlugins = exports.renderFileWithPlugins = function () {
  var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(file, template, siteData, eventBefore, eventAfter) {
    var renderedFile, _ref5, _ref6;

    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (!eventBefore) {
              _context2.next = 3;
              break;
            }

            _context2.next = 3;
            return _plugin2.default.processEventHandlers(eventBefore, file);

          case 3:
            renderedFile = file.render(template, siteData);

            if (!eventAfter) {
              _context2.next = 11;
              break;
            }

            _context2.next = 7;
            return _plugin2.default.processEventHandlers(eventAfter, file, renderedFile);

          case 7:
            _ref5 = _context2.sent;
            _ref6 = (0, _slicedToArray3.default)(_ref5, 2);
            file = _ref6[0];
            renderedFile = _ref6[1];

          case 11:
            return _context2.abrupt('return', [file, renderedFile]);

          case 12:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function renderFileWithPlugins(_x5, _x6, _x7, _x8, _x9) {
    return _ref4.apply(this, arguments);
  };
}();

/**
 * Combines renderFileWithPlugins and writeToDiskWithPlugins for conveinience.
 * @return {Promise}
 */


var renderAndWriteFileWithPlugins = exports.renderAndWriteFileWithPlugins = function () {
  var _ref7 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
    var _ref8,
        _ref9,
        file,
        renderedFile,
        _args3 = arguments;

    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return renderFileWithPlugins.apply(undefined, _args3);

          case 2:
            _ref8 = _context3.sent;
            _ref9 = (0, _slicedToArray3.default)(_ref8, 2);
            file = _ref9[0];
            renderedFile = _ref9[1];
            return _context3.abrupt('return', writeToDiskWithPlugins(file, renderedFile));

          case 7:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function renderAndWriteFileWithPlugins(_x10) {
    return _ref7.apply(this, arguments);
  };
}();

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _plugin = require('./plugin');

var _plugin2 = _interopRequireDefault(_plugin);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }