'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ErrorMessage = undefined;
exports.render = render;
exports.renderString = renderString;
exports.addFilter = addFilter;
exports.addBuiltinFilters = addBuiltinFilters;
exports.configure = configure;

var _nunjucks = require('nunjucks');

var _nunjucks2 = _interopRequireDefault(_nunjucks);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _url = require('../url');

var _url2 = _interopRequireDefault(_url);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var env = void 0;

/**
 * Render a template with given context variables.
 * @param {string} template  Template name, excluding the extension.
 * @param {object} variables Object of variables to interpolate in the
 *   template.
 * @return {string} Rendered template.
 */
/**
 * Methods for rendering a template.
 */
function render(template, variables) {
  if (!env) {
    return '';
  }

  return env.render(template + '.html', variables);
}

/**
 * Render a string template with given context variables.
 * @param {string} str  Template string.
 * @param {object} variables Object of variables to interpolate in the
 *   template.
 * @return {string} Rendered template.
 */
function renderString(str, variables) {
  if (!env) {
    return '';
  }

  return env.renderString(str, variables);
}

/**
 * Allow adding custom filters to the template engine.
 * @see {@link
 *   http://mozilla.github.io/nunjucks/api#custom-filters
 * @param {string} name Filter name.
 * @param {Function} func Filter function.
 * @param {boolean} async Whether the filter should be async.
 */
function addFilter(name, func) {
  var async = arguments.length <= 2 || arguments[2] === undefined ? false : arguments[2];

  env.addFilter(name, func, async);
}

/**
 * Adds built in filters to template renderer.
 */
function addBuiltinFilters() {
  [['date', function (date, momentTemplate) {
    return (0, _moment2.default)(date).format(momentTemplate);
  }], ['slug', function (str) {
    return _url2.default.slug(str);
  }], ['absolute_url', function (relativePath, basePath) {
    var base = basePath || '';
    var baseLength = base.length - 1;
    base = base[baseLength] === '/' ? base.substr(0, baseLength) : base;

    var input = relativePath || '';
    var inputLength = input.length - 1;
    if (input[inputLength] !== '/') {
      input = input + '/';
    }
    if (input[0] !== '/') {
      input = '/' + input;
    }

    return base + input;
  }], ['limit', function (arr, length) {
    return arr.slice(0, length);
  }]].forEach(function (filter) {
    addFilter.apply(null, filter);
  });
}

/**
 * Exposed method to configure template engine.
 * @param {Object} options Options object with following properties:
 *   @param {string|Array.<string>} paths Either an array of paths or a singular
 *     path that we can load templates from.
 *   @param {boolean} cacheTemplates Whether our template engine should cache
 *     its templates. Only set to false when in watch mode.
 */
function configure(_ref) {
  var paths = _ref.paths;
  var _ref$cacheTemplates = _ref.cacheTemplates;
  var cacheTemplates = _ref$cacheTemplates === undefined ? false : _ref$cacheTemplates;

  var fileSystemLoader = new _nunjucks2.default.FileSystemLoader(paths, {
    noCache: cacheTemplates
  });
  env = new _nunjucks2.default.Environment(fileSystemLoader, {
    autoescape: false
  });

  addBuiltinFilters();
}

var ErrorMessage = exports.ErrorMessage = {
  NO_TEMPLATE: 'template not found'
};