'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _isUndefined = require('lodash/isUndefined');

var _isUndefined2 = _interopRequireDefault(_isUndefined);

var _log = require('../log');

var _log2 = _interopRequireDefault(_log);

var _browserify = require('./processor/browserify');

var _browserify2 = _interopRequireDefault(_browserify);

var _less = require('./processor/less');

var _less2 = _interopRequireDefault(_less);

var _sass = require('./processor/sass');

var _sass2 = _interopRequireDefault(_sass);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var processors = {
  browserify: _browserify2.default,
  less: _less2.default,
  sass: _sass2.default
};

var Asset = function () {
  function Asset(assetType, assetConfig) {
    (0, _classCallCheck3.default)(this, Asset);

    /**
     * ID of asset, right now an alias to its type.
     * @type {string}
     */
    this.id = assetType;

    /**
     * What type of asset this is. css/js/font etc.
     * @type {string}
     */
    this.type = assetType;

    /**
     * The asset config object.
     * @type {Object}
     */
    this.config = assetConfig;

    // If this asset has a processor, instantiate the class that handles how to
    // process the asset.
    if (this.config.processor && this.config.processor.name) {
      var processorName = this.config.processor.name;
      var ProcessorClass = processors[processorName];

      if ((0, _isUndefined2.default)(ProcessorClass)) {
        var msg = 'Could not load processer \'' + processorName + '\'. Aborting.';
        _log2.default.error(msg);
        throw new Error(msg);
      }

      /**
       * Asset processor.
       * @type {Processor}
       */
      this.processor = new ProcessorClass(this);

      /**
       * Processed file.
       * @type {Object}
       */
      this._processedFile;
    }

    /**
     * Data accessible to templates.
     * @type {Object}
     */
    this.data = (0, _create2.default)(null);
  }

  (0, _createClass3.default)(Asset, [{
    key: 'process',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(destination) {
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (this.processor) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt('return');

              case 2:
                _context.prev = 2;
                _context.next = 5;
                return this.processor.process();

              case 5:
                this._processedFile = _context.sent;
                _context.next = 12;
                break;

              case 8:
                _context.prev = 8;
                _context.t0 = _context['catch'](2);

                _log2.default.error('Could not process file.');
                throw _context.t0;

              case 12:

                // Expose relative url to file.
                this.data.url = this._processedFile.destination.replace(destination, '');

              case 13:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[2, 8]]);
      }));

      function process(_x) {
        return _ref.apply(this, arguments);
      }

      return process;
    }()
  }, {
    key: 'write',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (!this.processor) {
                  _context2.next = 8;
                  break;
                }

                if (this._processedFile) {
                  _context2.next = 5;
                  break;
                }

                _log2.default.warn('Asset: ' + this.id + ' found no processed file when about ' + 'to write asset to disk. Processing now.');
                _context2.next = 5;
                return this.process();

              case 5:

                _fsExtra2.default.outputFileSync(this._processedFile.destination, this._processedFile.asset, 'utf8');
                _context2.next = 9;
                break;

              case 8:
                // If there's no processor then just copy the source to the destination.
                try {
                  _fsExtra2.default.copySync(this.config.source, this.config.destination);
                } catch (e) {
                  _log2.default.error('Asset: Could not copy \'' + this.type + '\' assets.');
                }

              case 9:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function write() {
        return _ref2.apply(this, arguments);
      }

      return write;
    }()
  }]);
  return Asset;
}();

exports.default = Asset;