'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _resolve = require('resolve');

var _resolve2 = _interopRequireDefault(_resolve);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _each = require('lodash/each');

var _each2 = _interopRequireDefault(_each);

var _map = require('lodash/map');

var _map2 = _interopRequireDefault(_map);

var _json = require('../json');

var _parse = require('../parse');

var _parse2 = _interopRequireDefault(_parse);

var _constants = require('../constants');

var CONSTANTS = _interopRequireWildcard(_constants);

var _asset = require('./asset');

var _asset2 = _interopRequireDefault(_asset);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Theme = function () {
  function Theme() {
    (0, _classCallCheck3.default)(this, Theme);

    /**
     * The current theme name.
     * @type {string}
     */
    this.name = '';

    /**
     * Raw object that holds the theme config object.
     * @type {Object}
     */
    this.config;

    /**
     * Collection of all assets for this theme.
     * @type {Object.<string, Asset>}
     */
    this.assets = (0, _create2.default)(null);

    /**
     * Data of asset names and paths that is exposed to templates.
     * @type {Object}
     */
    this.data = (0, _create2.default)(null);
  }

  /**
   * Set global config delegate function.
   * @param {Object} getConfig Config object.
   */


  (0, _createClass3.default)(Theme, [{
    key: 'setGetConfig',
    value: function setGetConfig(getConfig) {
      this._getConfig = getConfig;
    }

    /**
     * Find all theme.yml files in node_modules.
     * @param {string} directory Directory to load package.json from.
     * @return {Array.<string>} Array of paths to theme.yml files.
     */

  }, {
    key: '_getThemesFromPackageJson',
    value: function _getThemesFromPackageJson() {
      var directory = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];

      var yarnThemePackageNameRegex = /^yarn\-theme\-/;

      // All yarn packages that exist in our root package.json file.
      var yarnPackages = (0, _json.getYarnPackageNames)(directory);

      // Return only the path to the theme.yml file.
      return yarnPackages.reduce(function (packageThemes, packageName) {
        if (!yarnThemePackageNameRegex.test(packageName)) {
          return packageThemes;
        }

        // Use module's main field which should reference the config yaml file.
        var yamlPath = _resolve2.default.sync(packageName, {
          basedir: directory
        });

        packageThemes.push(yamlPath);

        return packageThemes;
      }, []);
    }
  }, {
    key: 'update',
    value: function update() {
      var _this = this;

      this.name = this._getConfig().get('theme');

      var configPath = this._getConfig().get('path');

      // Find all theme config files that might exist.
      var fileSystemYamls = _glob2.default.sync(configPath.themes + ('/**/' + CONSTANTS.YAML.THEME), { nodir: true });

      var moduleYamls = this._getThemesFromPackageJson(configPath.source);

      var allYamls = [].concat(fileSystemYamls, moduleYamls);

      var files = allYamls.reduce(function (allFiles, file) {
        var parsedFile = _parse2.default.fromYaml(_fsExtra2.default.readFileSync(file, 'utf8'));

        // If the found theme config file name matches our set theme name
        // then save it.
        if (parsedFile.name === _this.name) {
          // Save the directory where the correct theme file was found.
          parsedFile.path[CONSTANTS.KEY.SOURCE] = _path2.default.dirname(file);
          allFiles.push(parsedFile);
        }

        return allFiles;
      }, []);

      if (files.length === 0) {
        throw new Error('Did not find any theme named \'' + this.name + '\' installed.');
      } else if (files.length !== 1) {
        throw new Error('Found multiple themes with the same name: ' + files.join(','));
      }

      var parsedFile = files[0];

      // Save raw theme config.
      this.config = parsedFile;

      // Calculate absolute path of 'paths' keys.
      (0, _each2.default)(this.config.path, function (val, key) {
        if (key !== CONSTANTS.KEY.SOURCE) {
          (function () {
            // The root of the theme's destination is the site's destination.
            var rootPath = key === CONSTANTS.KEY.DESTINATION ? configPath.destination : _this.config.path.source;

            var keyValue = _this.config.path[key];

            // Support converting array of values to their absolute path.
            if ((0, _isArray2.default)(keyValue)) {
              _this.config.path[key] = keyValue.map(function (value) {
                return _path2.default.resolve(rootPath, value);
              });
            } else {
              _this.config.path[key] = _path2.default.resolve(rootPath, keyValue);
            }
          })();
        }
      });

      this._createAssets();
    }
  }, {
    key: '_createAssets',
    value: function _createAssets() {
      var _this2 = this;

      // Instantiate an Asset for every asset the theme configures.
      (0, _each2.default)(this.config.assets, function (assetConfig, assetType) {
        assetConfig.destination = _path2.default.resolve(_this2.config.path.destination, assetConfig.source);

        assetConfig.source = _path2.default.resolve(_this2.config.path.source, assetConfig.source);

        var asset = new _asset2.default(assetType, assetConfig);
        _this2.assets[asset.id] = asset;
      });
    }
  }, {
    key: 'read',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
        var _this3 = this;

        var pathDestination;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                pathDestination = this._getConfig().get('path.destination');

                // Have every asset write itself to the destination folder.

                _context.next = 3;
                return _promise2.default.all((0, _map2.default)(this.assets, function (assetProcessor) {
                  return assetProcessor.process(pathDestination);
                }));

              case 3:

                // Expose every asset's data onto the theme's data object.
                (0, _each2.default)(this.assets, function (assetProcessor) {
                  if (assetProcessor.data) {
                    _this3.data[assetProcessor.type] = assetProcessor.data.url;
                  }
                });

              case 4:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function read() {
        return _ref.apply(this, arguments);
      }

      return read;
    }()
  }, {
    key: 'write',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return _promise2.default.all((0, _map2.default)(this.assets, function (assetProcessor) {
                  return assetProcessor.write();
                }));

              case 2:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function write() {
        return _ref2.apply(this, arguments);
      }

      return write;
    }()
  }]);
  return Theme;
}();

exports.default = Theme;