'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _checksum = require('../checksum');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ProcessorBase = function () {
  function ProcessorBase(asset) {
    (0, _classCallCheck3.default)(this, ProcessorBase);

    /**
     * The asset we're processing.
     * @type {Asset}
     * @private
     */
    this._asset = asset;

    /**
     * Path from where to read the asset.
     * @type {string}
     */
    this.assetSource = this._asset.config.source;

    /**
     * Path of where to write the asset.
     * @type {string}
     */
    this.assetDestination = this._asset.config.destination;

    /**
     * What plugins to apply to the processor.
     * @type {Object}
     */
    this.plugins = this._asset.config.processor.plugins;

    /**
     * Whether we should hash the file name.
     * @type {boolean}
     */
    this.shouldHash = this._asset.config.processor.hash;
  }

  /**
   * Public API that is used to process the asset.
   * @return {Promise} Returns a promise that resolves with the destination
   *  and actual content of the processed asset.
   */


  (0, _createClass3.default)(ProcessorBase, [{
    key: 'process',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
        var asset, destination, extension;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return this._getFile();

              case 2:
                asset = _context.sent;
                destination = this._getDestination(asset);


                if (this.shouldHash) {
                  this.checksum = (0, _checksum.createChecksum)(asset).slice(0, 10);

                  // Get destination extension, i.e. `.css`.
                  extension = _path2.default.extname(destination);

                  // Append checksum to path.

                  destination = destination.replace(new RegExp(extension + '$'), '-' + this.checksum + extension);
                }

                return _context.abrupt('return', {
                  destination: destination,
                  asset: asset
                });

              case 6:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function process() {
        return _ref.apply(this, arguments);
      }

      return process;
    }()
  }, {
    key: '_getDestination',
    value: function _getDestination() {
      return this.assetDestination;
    }
  }]);
  return ProcessorBase;
}();

exports.default = ProcessorBase;